@@ -267,6 +267,8 @@ class FriendListRequest(WebQQRequest):
         logger.info("加载好友信息 {0!r}".format(friends))
         logger.info(data)
         self.hub.load_next_request(GroupListRequest())
+        self.hub.load_next_request(FriendStatusRequest())
+        self.hub.load_next_request(DiscuListRequest())
         self.hub.load_next_request(FriendListRequest(delay=3600, first=False))
 
 
@@ -276,6 +278,26 @@ warnings.warn("In next version we will rename twqq.requests.FreindInfoRequest "
               "to twqq.requests.FriendListRequest")
 
 
+class FriendStatusRequest(WebQQRequest):
+
+    """ 获取在线好友状态
+    """
+
+    url = "https://d.web2.qq.com/channel/get_online_buddies2"
+
+    def init(self):
+        self.params = {"clientid": self.hub.clientid,
+                       "psessionid": self.hub.psessionid,
+                       "t": int(time.time() * 1000)}
+        self.headers.update(Referer=const.D_REFERER)
+
+    def callback(self, response, data):
+        logger.info(u"加载好友状态信息: {0!r}".format(data))
+        if isinstance(data, dict) and data.get("retcode") == 0:
+            for item in data.get('result', []):
+                self.hub.get_friends().set_status(**item)
+
+
 class GroupListRequest(WebQQRequest):
 
     """ 获取群列表
@@ -376,11 +398,10 @@ class PollMessageRequest(WebQQRequest):
                 return
 
             logger.info(u"获取消息: {0!r}".format(data))
+            self.hub.load_next_request(PollMessageRequest())
             self.hub.dispatch(data)
         except Exception as e:
             logger.error(u"消息获取异常: {0}".format(e), exc_info=True)
-        finally:
-            self.hub.load_next_request(PollMessageRequest())
 
 
 class SessGroupSigRequest(WebQQRequest):
@@ -497,11 +518,35 @@ class DiscuListRequest(WebQQRequest):
         self.headers.update(Referer=const.S_REFERER)
 
     def callback(self, resp, data):
+        logger.info(u"获取讨论组列表: {0!r}".format(data))
         if data.get("retcode") == 0:
             self.hub.set_discu(data.get("result", {}))
+            dids = self.hub.get_discu().dids
+            for did in dids:
+                self.hub.load_next_request(DiscuInfoRequest(did))
 
 
-class DiscuInfo(WebQQRequest):
+class QQNumberRequest(WebQQRequest):
+    """ 查看好友QQ号码
+    """
+    url = "http://s.web2.qq.com/api/get_friend_uin2"
+
+    def init(self, uin):
+        self.params = {"code": "", "t": time.time() * 1000,
+                       "tuin": uin, "type": 0,
+                       "verifysession": "",
+                       "vfwebqq": self.hub.vfwebqq}
+        self.headers.update(Referer=const.S_REFERER)
+
+    def callback(self, response, data):
+        if isinstance(data, dict) and data.get("retcode") == 0:
+            r = data.get('result')
+            uin = r.get("uin")
+            account = r.get("account")
+            self.hub.get_friends().set_account(uin, account)
+
+
+class DiscuInfoRequest(WebQQRequest):
     """ 获取讨论组详细信息
 
     :param did: 讨论组id
@@ -633,6 +678,38 @@ class AcceptVerifyRequest(WebQQRequest):
             logger.info(u"添加 {0} 失败".format(self.qq_num))
 
 
+class FileRequest(WebQQRequest):
+    """ 下载传送的文件
+
+    :param guid: 文件名
+    :param lcid: 会话id
+    :param to_uin: 发送人uin
+    """
+    url = "http://d.web2.qq.com/channel/get_file2"
+
+    def init(self, guid, lcid, to, callback=None):
+        self.params = {"clientid": self.hub.clientid,
+                       "psessionid": self.hub.psessionid,
+                       "count": 1, "time": int(time.time() * 1000),
+                       "guid": guid, "lcid": lcid, "to": to}
+        self.headers.update(Referer="http://web2.qq.com/webqq.html")
+        self.headers.pop("Origin", None)
+        self.fname = guid
+        self._cb = callback
+
+    def callback(self, response, data):
+        """ 应该在客户端通过::
+
+            @register_request_handler(FileRequest)
+            def callback(resp, data):
+                pass
+
+        来实现文件的保存
+        """
+        if self._cb:
+            self._cb(self.fname, response.body)
+
+
 FirstRequest = LoginSigRequest
 
 
@@ -665,7 +742,7 @@ def group_message_handler(func):
         gcode = value.get("group_code")
         uin = value.get("send_uin")
         contents = value.get("content", [])
-        content = self.handle_qq_msg_contents(contents)
+        content = self.handle_qq_msg_contents(uin, contents, gcode)
         uname = self.get_group_member_nick(gcode, uin)
         return uname, content, gcode, uin, message
 
@@ -686,7 +763,7 @@ def buddy_message_handler(func):
         value = message.get("value", {})
         from_uin = value.get("from_uin")
         contents = value.get("content", [])
-        content = self.handle_qq_msg_contents(contents)
+        content = self.handle_qq_msg_contents(from_uin, contents)
         return from_uin, content, message
     return _register_message_handler(func, args_func, "message")
 
@@ -720,7 +797,7 @@ def sess_message_handler(func):
         id_ = value.get("id")
         from_uin = value.get("from_uin")
         contents = value.get("content", [])
-        content = self.handle_qq_msg_contents(contents)
+        content = self.handle_qq_msg_contents(from_uin, contents)
         return id_, from_uin, content, message
 
     return _register_message_handler(func, args_func, "sess_message")
@@ -756,12 +833,57 @@ def discu_message_handler(func):
 
     def args_func(self, message):
         value = message.get("value")
-        content = self.handle_qq_msg_contents(value.get("content", []))
-        return (value.get("did"), value.get("send_uin"), content, message)
+        from_uin = value.get("send_uin")
+        did = value.get("did")
+        content = self.handle_qq_msg_contents(
+            from_uin, value.get("content", []), did, 1)
+        return (did, from_uin, content, message)
 
     return _register_message_handler(func, args_func, "discu_message")
 
 
+def file_message_handler(func):
+    """ 装饰处理文件消息的函数
+
+    处理函数应接受
+        from_uin        文件发送人
+        to_uin          文件接收人
+        lcid            文件sessionid (此处为 session_id 字段)
+        guid            文件名称 (此处为 name 字段)
+        is_cancel       是否是取消发送
+        source          消息源包
+    """
+    def args_func(self, message):
+        value = message.get("value", {})
+        return (value.get("from_uin"), value.get("to_uin"),
+                value.get("session_id"), value.get("name"),
+                value.get("cancel_type", None) == 1,
+                message)
+    return _register_message_handler(func, args_func, "file_message")
+
+
+def offline_file_message_handler(func):
+    """ 装饰处理离线文件的函数
+
+    处理函数应接收
+        from_uin        文件发送人
+        to_uin          文件接收人
+        lcid            文件sessionid (此处为 session_id 字段)
+        count           离线文件数量
+        file_infos      文件信息
+        source          消息源包
+    """
+    warnings.warn(u"WebQQ的离线消息并不可靠, 对方可能发送, 但是收不到")
+
+    def args_func(self, message):
+        value = message.get("value", {})
+        return (value.get("from_uin"), value.get("to_uin"),
+                value.get("lcid"), value.get("count"),
+                value.get("file_infos"), message)
+
+    return _register_message_handler(func, args_func, "filesrv_transfer")
+
+
 def check_request(request):
     """ 检查Request参数是否合法, 并返回一个类对象
     """
